"""
Test if main domain of cluster is resolvable at all authorative nameservers,
and also external ones, and check if all nameservers return the same result.
"""

import os
import dns.resolver
import pytest
import tld

@pytest.mark.testinfra
@pytest.mark.dns
def test_dns(host):
    """Get domain from ansible vars, resolve it against all nameservers and
    check if all results are the same.
    """

    def test_nameserver(nameserver: str, domain: str) -> str:
        """Resolve domain at given nameserver."""
        my_resolver = dns.resolver.Resolver()
        my_resolver.nameservers = [nameserver]
        answer = my_resolver.resolve(domain)
        addr = answer[0].address
        print(f"{domain} @{nameserver}: {addr}")
        return addr

    # Get ansible domain
    ansible_vars = host.ansible.get_variables()
    domain = ansible_vars['domain']
    expected_ip = ansible_vars['ansible_host']

    # Get FLD and authorative nameservers for domain
    fld = tld.get_fld(domain, fix_protocol=True)
    authorative_nameservers_raw = dns.resolver.resolve(fld, 'NS')
    authorative_nameservers = []
    for nameserver in authorative_nameservers_raw:
        ips = dns.resolver.resolve(nameserver.to_text())
        authorative_nameservers.append(ips[0].address)

    system_nameservers = dns.resolver.get_default_resolver()
    external_nameservers = ['1.1.1.1', '8.8.8.8']

    results = []
    print('\nTesting authorative nameservers:')
    for i in authorative_nameservers:
        results.append(test_nameserver(i, domain))
        results.append(test_nameserver(i, "*." + domain))

    print('\nTesting system nameservers:')
    for i in system_nameservers.nameservers:
        results.append(test_nameserver(i, domain))
        results.append(test_nameserver(i, "*." + domain))

    print('\nTesting external nameservers')
    for i in external_nameservers:
        results.append(test_nameserver(i, domain))
        results.append(test_nameserver(i, "*." + domain))

    unique = set(results)
    print(f'\nResults: {unique}')
    assert len(unique) == 1
    assert list(unique)[0] == expected_ip


if __name__ == "__main__":
    test_dns('localhost')
